/++
 + Firebird接続ライブラリ Version: 0.6.1
 +
 + 拡張機能モジュール
 +
 + Copyright(C) 2009-2011 Takuya Kurosawa
 +/
module jp.ku6.firebird.extension;

import core.atomic;
import core.thread;
import jp.ku6.firebird.core;

/++
 + 貸し出せるコネクションが無い場合
 +/
class OutOfStockException : Exception
{
	/++
	 + コンストラクタ
	 +
	 + Params:
	 +  message = メッセージ
	 +/
	this(string message)
	{
		super(message);
	}
}

/++
 + シャットダウン時失敗時の例外
 +/
class ShutdownException : Exception
{
	/++
	 + コンストラクタ
	 +
	 + Params:
	 +  message = メッセージ
	 +/
	this(string message)
	{
		super(message);
	}
}

/++
 + コネクションプール
 +
 + 指定データベースへの接続を指定した本数プールする
 + 接続が必要な場合はlendで取得する
 + 取得した接続は必ずcollectで返却すること
 +/
shared
class ConnectionPool
{
	/++
	 + コンストラクタ
	 +
	 + Params:
	 +  database = データベース
	 +  account = データベースアカウント
	 +  password = データベースパスワード
	 +  capacity = 最大接続数
	 +/
	this(string database, string account, string password, uint capacity)
	{
		database_ = database;
		account_ = account;
		password_ = password;
		lentFlags_ = new shared(uint)[capacity];
		connections_ = new shared(Connection)[capacity];
	}

	/++
	 + 接続を貸し出す
	 +
	 + 返されるConnectionはattach済み
	 + 使用側は、取得したConnectionを必ずcollectに返却すること
	 +
	 + Params:
	 +  retry = 接続が残って居なかった場合のリトライ回数(デフォルトは0)
	 +  sleep = リトライ前に待つ時間 100ナノ秒単位(デフォルトは10ミリ秒)
	 +
	 + Returns:
	 +  データベース接続
	 +
	 + Throws:
	 +  RuntimeError = 接続時のエラー
	 +  OutOfStockException = 取得出来る接続が無い場合
	 +/
	Connection lend(uint retry = 0, uint sleep = 100000)
	{
		for (;;)
		{
			shared(uint)* lentFlags = lentFlags_.ptr;
			size_t length = lentFlags_.length;
			for (size_t i; i < length; ++i)
			{
				if (lentFlags[i] == 0 && cas(&lentFlags[i], 0u, 1u))
				{
					if (connections_[i] is null)
						connections_[i] = new shared(Connection)(database_, account_, password_);

					// 取得したスレッドのみの使用が保証されるのでsharedを外してもよい
					Connection connection = cast(Connection)connections_[i];
					if(!connection.isAttached())
					{
						try
						{
							connection.attach();
						}
						catch (RuntimeError re)
						{
							// 取得した接続は返却して、他のを探す
							lentFlags_[i] = 0;
							continue;
						}
					}

					return connection;
				}
			}

			if (retry == 0)
				throw new OutOfStockException("Connection is out of stock.");
			else
			{
				--retry;
				Thread.sleep(sleep);
			}
		}
	}

	/++
	 + 接続を受け取る
	 +
	 + Params:
	 +  connection = データベース接続
	 +/
	void collect(Connection connection)
	{
		shared(Connection)* connections = connections_.ptr;
		size_t length = connections_.length;
		for (size_t i; i < length; ++i)
		{
			if (connection is connections[i])
			{
				lentFlags_[i] = 0;
				return;
			}
		}
	}

	/++
	 + 終了処理
	 +
	 + アプリケーション終了時に接続の切断処理を行う
	 +
	 + Params:
	 +  retry = 接続が使用中だった場合のリトライ回数(デフォルトは0)
	 +  sleep = リトライ前に待つ時間 100ナノ秒単位(デフォルトは10ミリ秒)
	 +
	 + Throws:
	 +  RuntimeError = 切断時のエラー
	 +  ShutdownException = コネクションがまだ使用中の場合
	 +/
	void shutdown(uint retry = 0, uint sleep = 100000)
	{
		for (;;)
		{
			shared(uint)* lentFlags = lentFlags_.ptr;
			size_t length = lentFlags_.length;

			bool complete = true;
			for (size_t i; i < length; ++i)
			{
				if (lentFlags[i] == 1)
					complete = false;
				else if (lentFlags[i] == 0)
				{
					if (cas(&lentFlags[i], 0u, 2u))
						(cast(Connection)connections_[i]).detach();
					else
						complete = false;
				}
			}

			if (retry == 0)
				throw new ShutdownException("Connection is already used.");
			else
			{
				--retry;
				Thread.sleep(sleep);
			}
		}
	}

private:

	// 接続先のデータベース
	string database_;

	// データベースアカウント
	string account_;

	// データベースパスワード
	string password_;

	// コネクション使用フラグ
	uint[] lentFlags_;

	// コネクションプール
	Connection[] connections_;
}
