/++
 + FastCGIフレームワーク Raptan Version: 0.3.3
 +
 + フィルタモジュール
 +
 + Copyright(C) 2009-2011 Takuya Kurosawa
 +/
module jp.ku6.raptan.filter;

import std.string;
import std.path;
import jp.ku6.raptan.core;

/++
 + リクエストの拡張子に対してContent-Typeを指定するフィルタ
 +
 + リクエストの拡張子と異なるContent-Typeを出す場合は
 + Action内でフィルタの出力を上書きしてください
 +/
class ContentTypeFilter : Filter
{
	/++
	 + Content-Typeを追加
	 +
	 + Params:
	 +  extension = 拡張子 (大文字小文字区別無し)
	 +  contentType = Content-Type
	 +/
	void add(string extension, string contentType)
	{
		contentTypes_[toLower(extension)] = contentType;
	}

	override
	void execute(Request request, Response response)
	{
		// リクエストパスから拡張子を取得
		string scriptName = request.getEnvironmentVariable("SCRIPT_NAME");

		size_t start = 0;
		for (size_t i; i < scriptName.length; ++i)
		{
			if (scriptName[i] == '.')
				start = i + 1;
		}

		string extension = 0 < start && start < scriptName.length ? scriptName[start .. $] : "";


		// 小文字に寄せる
		extension = toLower(extension);


		// Content-Typeを検索して、有ったら設定
		const(string)* type = extension in contentTypes_;
		if (type !is null)
			response.contentType = *type;

		executeAfterFilters(request, response);
	}

private:

	// キーが拡張子、値がContentType
	string[string] contentTypes_;
}

/++
 + 受け付けるリクエストメソッドを制限する
 +/
class RequestMethodFilter : Filter
{
	/++
	 + コンストラクタ
	 +
	 + 使用方法の例
	 + auto filter = new RequestMethodFilter(Request.Method.GET | Request.Method.POST);
	 +
	 + Params:
	 +  allowMethods = 受け付けるリクエストメソッドの値をORで連結した値
	 +/
	this(int allowMethods)
	{
		allowMethods_ = allowMethods;
	}

	override
	void execute(Request request, Response response)
	{
		if ((request.method & ~allowMethods_) != 0)
		{
			response.statusCode = Response.StatusCode.METHOD_NOT_ALLOWED;
			return;
		}
		else
			executeAfterFilters(request, response);
	}

private:

	// 受け付けるリクエストメソッドの値をORで連結した値
	int allowMethods_;
}
